@@ -7,7 +7,7 @@
  
  @section LICENSE
  
- Copyright (c) 2008-2016 OpenShot Studios, LLC
+ Copyright (c) 2008-2018 OpenShot Studios, LLC
  (http://www.openshotstudios.com). This file is part of
  OpenShot Video Editor (http://www.openshot.org), an open-source project
  dedicated to delivering high quality video editing and animation solutions
@@ -86,7 +86,7 @@ class FilesTreeView(QTreeView):
             # Add edit title option (if svg file)
             selected_file_id = self.win.selected_files[0]
             file = File.get(id=selected_file_id)
-            if file.data.get("path").endswith(".svg"):
+            if file and file.data.get("path").endswith(".svg"):
                 menu.addAction(self.win.actionEditTitle)
                 menu.addAction(self.win.actionDuplicateTitle)
                 menu.addSeparator()
@@ -272,15 +272,17 @@ class FilesTreeView(QTreeView):
         # Reset list of ignored image sequences paths
         self.ignore_image_sequence_paths = []
 
-        # log.info('Dropping file(s) on files tree.')
         for uri in event.mimeData().urls():
-            file_url = urlparse(uri.toString())
-            if file_url.scheme == "file":
-                filepath = file_url.path
-                if sys.platform == "win32":
-                    filepath = filepath[1:] # Remove / at beginning of path (just for Windows)
-                if os.path.exists(filepath.encode('UTF-8')) and os.path.isfile(filepath.encode('UTF-8')):
-                    log.info('Adding file: {}'.format(filepath))
+            log.info('Processing drop event for {}'.format(uri))
+            filepath = uri.toLocalFile()
+            if os.path.exists(filepath) and os.path.isfile(filepath):
+                log.info('Adding file: {}'.format(filepath))
+                if ".osp" in filepath:
+                    # Auto load project passed as argument
+                    self.win.OpenProjectSignal.emit(filepath)
+                    event.accept()
+                else:
+                    # Auto import media file
                     if self.add_file(filepath):
                         event.accept()
 
